#include <iostream>
#include <iomanip>

#include "./common.h"
#include "./types.h"
#include "./geom.h"

using namespace std;

void PrintMatrix(GLint matrix) {
  GLfloat m[16];
  glGetFloatv(matrix, m);
  cout.precision(2);
  int w = 6;
  for (int i = 0; i < 4; ++i) {
    cout << setprecision(2) << setw(w) << m[i] << " " 
         << setprecision(2) << setw(w) << m[i+4] << " "
         << setprecision(2) << setw(w) << m[i+8] << " "
         << setprecision(2) << setw(w) << m[i+12] << " "
         << endl;
  }
  cout << endl;
}

void PrintMatrix() {
  PrintMatrix(GL_MODELVIEW_MATRIX);
}

void LoadMatrix(GLfloat* m) {
  // transpose to column-major
  for (int i = 0; i < 4; ++i) {
    for (int j = i; j < 4; ++j) {
      swap(m[i*4+j], m[j*4+i]);
    }
  }
  glLoadMatrixf(m);
}

void MultMatrix(GLfloat* m) {
  // transpose to column-major
  for (int i = 0; i < 4; ++i) {
    for (int j = i; j < 4; ++j) {
      swap(m[i*4+j], m[j*4+i]);
    }
  }
  glMultMatrixf(m);
}

void Display() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  // glFrustum(-2, 2, -1, 1, 17, 23);
  gluPerspective(25, 1, 17, 23);
  PrintMatrix(GL_PROJECTION_MATRIX);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluLookAt(0, 0, 20,
            0, 0, 0,
            0, 1, 0);

  glLineWidth(4);
  DisableLighting();
  DrawAxis();

  glColor3f(0, 0, 0);
  glutWireCube(2);

  glFlush();
  glutSwapBuffers();
}

void Display1() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  // What is rendered?
  //   A) a cube
  //   B) a square
  //   C) disconnected lines
  //   D) none of the above
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glFrustum(-2, 2, -2, 2, 18, 22);
  PrintMatrix(GL_PROJECTION_MATRIX);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluLookAt(0, 0, 20,
            0, 0, 0,
            0, 1, 0);

  glLineWidth(4);
  DisableLighting();
  DrawAxis();

  glColor3f(0, 0, 0);
  glutWireCube(2);

  glFlush();
  glutSwapBuffers();
}

void Display2() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  // What is rendered?
  //   A) a cube
  //   B) a square
  //   C) disconnected lines
  //   D) none of the above
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  // changed from glFrustum(-2, 2, -2, 2, 18, 22);
  glFrustum(-2, 2, -2, 2, 21, 23);
  // gluPerspective(25, 1, 17, 23);
  PrintMatrix(GL_PROJECTION_MATRIX);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluLookAt(0, 0, 20,
            0, 0, 0,
            0, 1, 0);

  glLineWidth(4);
  DisableLighting();
  DrawAxis();

  glColor3f(0, 0, 0);
  glutWireCube(2);

  glFlush();
  glutSwapBuffers();
}

void Display3() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  // What is rendered?
  //   A) a cube
  //   B) a square
  //   C) a really tall box
  //   D) a really wide box
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  // changed from glFrustum(-2, 2, -2, 2, 18, 22);
  glFrustum(-20, 20, -2, 2, 18, 22);
  PrintMatrix(GL_PROJECTION_MATRIX);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluLookAt(0, 0, 20,
            0, 0, 0,
            0, 1, 0);

  glLineWidth(4);
  DisableLighting();
  DrawAxis();

  glColor3f(0, 0, 0);
  glutWireCube(2);

  glFlush();
  glutSwapBuffers();
}

void Display4() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  // What is rendered?
  //   A) a cube
  //   B) a square
  //   C) a really tall box
  //   D) a really wide box
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  // changed from glFrustum(-2, 2, -2, 2, 18, 22);
  gluPerspective(25, 1, 18, 22);
  PrintMatrix(GL_PROJECTION_MATRIX);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluLookAt(0, 0, 20,
            0, 0, 0,
            0, 1, 0);

  glLineWidth(4);
  DisableLighting();
  DrawAxis();

  glColor3f(0, 0, 0);
  glutWireCube(2);

  glFlush();
  glutSwapBuffers();
}

void Display5() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  // What is rendered?
  //   A) a cube
  //   B) a square
  //   C) a really tall box
  //   D) a really wide box
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  // changed from gluPerspective(25, 1, 18, 22);
  gluPerspective(25, 0.5, 18, 22);
  PrintMatrix(GL_PROJECTION_MATRIX);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluLookAt(0, 0, 20,
            0, 0, 0,
            0, 1, 0);

  glLineWidth(4);
  DisableLighting();
  DrawAxis();

  glColor3f(0, 0, 0);
  glutWireCube(2);

  glFlush();
  glutSwapBuffers();
}

